/*
  Program:  servo record playback40.c

  Status:   Good 
                - Timing re-trimmed OK for faster reads/writes
                - Replaced 4 Byte read with 32 Byte read and write; Updated Mode 7, mode 2, Mode 3 - Sped up routines!
                - Added edit portion by sample, servo, and value; Used r+ to overwrite specific cell in file
                - Added Line Edit Mode 9; Using Fseek and other file operations
                - Added new Cog for ADC in background - speeds up conversion 
                - Adjust all mode timing to accomodate 20 Hz update rate;  Mode 2 still slow by 11 mS per sample                      
                - 4 ADCs; Servo bank logic; getStr replaces scanf; 8th servo new library; Cut Code size


  Author:   Brian Lincoln (lightman)
  Date:     6/28/14

  Description:
      1) To playback and control multiple servos by reading commands from SD card and outputting at ~20 Hz rate
      2) To record 0 deg to 180 deg Servo movements through puppeteering onto SD card at 20 Hz Rate
      3) Uses 4 AD convertors for ch 1 through 4 in creating new show and editing is done by ADC
      4) ADC done in background with separate cog...values always ready

  Revision Updates:
  6/28/14 BTL - Original from tutorial program on SD card
              - Limit values between 0 to 1800 (0 deg to 180 deg)
              - Use Servo Library commands and servo angle commands for both record and playback
              - Add File names entry and loading
              - Used scan instead of Scan to read in file name.  Scan doesnt work right... no print out
              - Added Main Menu with 3 modes - along with Change File Name mode
  6/29/14 BTL - Added Append Feature to read SD card... change entry and then write back to card in single step
              - Added temp.txt working file to store new appended data; Updated terminal interface
              - Changed Menu ---- Append to Edit
              - Added copying file to new file name to complete Edit Routine (not affecting old file)
              - Added Limits to Edit mode data entry (0 to 1800)
              - Added fwrite in edit mode if no new data entered
  6/30/14 BTL - Added Record Potentiometer mode - Use ADC3 with 0 to 3.3V signal to input position
                then output to servo while recording on SD Card
              - Trimmed timing for print statement - Reduced Pause time from 50mS to 45mS and 44mS
              - Added File name to Main Menu header
              - Rescaled A/D convertor to new range of 0 to 1800 rather than old range of 350 to 1252 corresponding
                to a 0V to 3.3V input range to A/D Convertor
              - Added Mode 6 Setting limits of Servos
              - Created Servo Array for 8 servos
              - Print out servo limits with Exit criteria
              - Added Saving of Profile to SD Card
              - Added Reading of profile at startup
              - Limited Read of servo file with servo limit profile
              - Added Show time variable for recording with Pot 
  7/1/14  BTL - Updated menu text; Cleaned up flow for Set Limits
              - Added visual "Ready Set Go" start timer to Record mode
              - Added full 8 channel capability; Added new show mode
              - Updated Record Pot mode so it works per channel
              - Re wrote the read back mode for all 8 channels with display
                Servo 8 is dummy code on commanding servo since C library doesnt support it at present
              - Added Temp File re-write to new file in Record Pot/Ch mode
              - Updated Record Pot/ch to write directly to new filename
              - Made Servo Limits move the that particular servo (except #8)
              - Disabled Edit Mode (mode 5) for now; It overwrites file
              - Inserted Warnings on Mode 1 Record Key... overwrites file
              - Reformated Read list output with %5d or 5 places per field (on debug screen)
              - Reformated the Mode 2 Record Pot list output with %5d also
              - Reformated Servo Limit list output with %5d
              - Added Listing decision in Playback mode; listing or no listing on screen
              - Reduced Pause in Record Pot to 5mS (from 44mS)and Read Playback to 40mS
              - Updated Main Menu with extra dashed line
  7/3/14  BTL - Added Manual mode to play with servos; Replaced Recod Key Mode
  7/4/14  BTL - Added Rest mode to limits; Created new servorest array
              - Added Rest Mode (mode 8); Added to start up routine - Set Servos to Rest
              - Added Rest to Show Creation Mode
              - Updated intializing arrays to 900 cmd and rest; Lower limit to 50
              - Added Profile file name loading protection at start up... allowed new profile
                Loads default rest positions if new profile filename 
              - Removed confirmation of string names
              - Updated Mode 7 New Show use any servo... not just servo1
              - Added Profile filename change to mode 4 with file name error checking
              - Changed text in Mode 3 to playback instead of read back
  7/5/14  BTL - Added 4 A/D convertors in New Show mode.... Trimmed up timing and commented out print
                Originated from Servo 'Record Playback 34.c' - then modified for updated New Show w/ 4 A/D convertors
                Created New parallel version Servo 'Record Playback34 AD1.c'
              - Added Pot Reversal to Profile and to all modes (1800 - servocmd)
  7/6/14  BTL - Added Pot to ADC mapping - Removed Edit Single Step; Implemented Manual mode
                Continued with record mode; Had to reduce code size... running into run time errors prob due to low mem.
              - Added New Show updates with ADC assignment logic; Wathcing memory size closely (under 32K)
              - Updated Manual mode to include all four ADC
  7/8/14  BTL - Added 8th servo driven by new update C Library servo.h (now supports 14 servos)
              - Compiler Hub memory exceeded with new libraries... need to reduce code by 4,824 bytes
              - Replaced all scanf with scan to save code space - Now it compiles again with latest library updates
                BUT... input from keyboard no longer recognized by program.  scan and scanf perform differently
              - Using   getStr(servostr, 14); instead of scanf; Compiler working code size reduced    
  7/9/14  BTL - Switched versions: Renamed to servo record playback35 from servo record playback34 AD8.c  
              - Added Servo Bank (4 ch) Logic; Starting address of servo bank 1 to 5 entered allowing groups (banks)
                of 4 servos and ADC to be matched for new show and record a channel
              - Added bankstart variable with getStr from keyboard 
              - Add bank to mode 2 record.. so all recording modes support bank recording
              - Cleaned up recording modes, re-trimmed to max speed
  7/10/14 BTL - Accomodated slower playback to match recording timing... instead of 20 Hz frame rate, use 16.6667 Hz
                Frame is group of servo commands sent to servo_angle() function per time period. 
                Note: servo_angle() function updates actual PWM to servos at 20mS period or 50Hz update rate even though
                      Servo Recorder only issues commands at 16.6667 Hz or 60 mS period
              - Added New Cog for ADC processes to perform in background; Volatile array created to pass values
                from new cog to main cog;  Updated all the affected modes with new assignment statements
              - Increased speed for all modes.  Mode 2 record still 11 mS too long per sample time
              - Experimented w/removing pot rev from mode 2 record to save time; no big improvement; left them in
              - Remove i record counter in mode 2 record...little help
              - Fixed print format statement in Mode 2 to print out new file name... 
                was missing %d for s variable... so I removed them
  7/13/14 BTL - Added Edit Line mode 9; Used Fseek and other file operators instead of fread to move through 
                SD file quickly
              - Added escape from mode 9 before Edit and Overwrite performed...just hit enter with nothing entered
              - Removed print of '.txt' from opening file input request
  7/15/14 BTL - Replaced 4 byte read and writes with 32 Byte read and writes to speed up routines... it worked!
                Updated Mode 2, 3, and 7 so far... Need to adjust timing pauses to get back to proper 20Hz Rate
                Trimmed all modes for timing updates due to faster reads and writes



  Propeller I/O:

  Pin   Description
  0     Spare
  1     Spare
  2     Spare
  3     Spare
  4     Spare
  5     Spare
  6     Spare
  7     Spare
  8     Spare
  9     Spare 
  10    Servo Motor8 (Now in use! with updated C Library)
  11    Servo Motor7
  12    Servo Motor6
  13    Servo Motor5
  14    Servo Motor4
  15    Servo Motor3
  16    Servo Motor2
  17    Servo Motor1
  18 to 31 are reserved for SD card, Audio, DAC, LEDs, etc




*/

#include "simpletools.h"                      // Include simpletools header
#include "servo.h"
#include "adcDCpropab.h"                      // Include adcDCpropab

void ADCbackground();                             // Forward declaration of new ADC cog
int *cog;                                         // For storing process ID
volatile int ADCvalue[] = {900, 900, 900, 900};   // Declare ADCvalue array for both cogs


//----------- Variable / Constants Declarations --------------------------------------------
char servostr [15];                           // Servo Filename - Includes null character... usable space is 14 characters
char profilestr [15];                         // Profile Filename for servo project
char servonewstr [15];                        // New servo filename

int n, status, mode, tmp, s;                  // Number & status variables & mode, Servo number

int showtime = 300;                           // Establish show length in 50mS samples
int mid = 900;                                // Midpoint of servos for new show initialization
int bankstart = 1;                                 // Servo Bank starting address (1 to 5)

int servocmd[] =  {0,  900,  900,  900,  900,  900,  900,  900,  900}; // Est. array & Set Initial Cond for 8 servo angles
int servolow[] =  {0,   50,   50,   50,   50,   50,   50,   50,   50}; // Est. array & Set Lower limit defaults for 8 servos
int servohigh[] = {0, 1750, 1750, 1750, 1750, 1750, 1750, 1750, 1750}; // Est. array & Set upper limit defaults for 8 servos
int servorest[] = {0,  900,  900,  900,  900,  900,  900,  900,  900}; // Est. array & Set Rest Positions for 8 servo angles
int potrev[] =    {0,    0,    0,    0,    0,    0,    0,    0,    0}; // Est. Array & set reversal flag (0 = normal,  1 = reverse)
int servoADC[] =  {0,    3,    2,    1,    0,    3,    2,    1,    0};  // Est. array and assign ADC 0, 1, 2, & 3 to a servo in servo adc as default; 9 = no assignment

     
int DO = 22, CLK = 23, DI = 24, CS = 25;      // SD card pins on Propeller BOE



//------------- MAIN FUNCTION ----------------------------------------------------------------
int main(void)                                // main function
{
  cog = cog_run(&ADCbackground, 10);          // Run ADC in other cog in background
  sd_mount(DO, CLK, DI, CS);                  // Mount SD card

  FILE* fp;                                   // Open a pointer for SD card Files
  FILE* fptmp;                                // Open 2nd temporary pointer for working file
  FILE* fpprof;                               // Open 3rd file pointer for Profile

//------------- Initialize -------------------------------------------------------------------
  for(int i = 1; i <= 8; i++)                 // Counter for 7 servos; 8th is OK now
  {
    servo_angle((18-i), mid);                 // Initialize all Servos to midpoint
  } //end for

  adc_init(21, 20, 19, 18);                   // CS=21, SCL=20, DO=19, DI=18

//----------- Print out Banner Header --------------------------------------------------------
  print("----- Servo Recorder ------\n");
  print("          Ver 40\n\n");


  //-------- Enter Servo Command File Names -----------------------------------------------------
  print("Enter Servo filename: ");
  getStr(servostr, 14);                      // Scan in string from debug terminal (have to hit enter) <= 14 characters  


  //-------- Enter Profile File Names & Load file -----------------------------------------------------

  print("Enter Profile filename: ");
  getStr(profilestr, 14);                    // Scan in string from debug terminal (have to hit enter) <= 14 characters  


  fpprof = fopen(profilestr, "r");           // Open a file for read
  if(fpprof)                                 // if file exists (fpprof = 1) then proceed to load
  {
    for(int i = 1; i <= 8; i++)              //Readin Profile
    {
      fread(&servolow[i], 4, 1, fpprof);     // read profile from SD card New File
      fread(&servohigh[i], 4, 1, fpprof);
      fread(&servorest[i], 4, 1, fpprof);
      fread(&potrev[i], 4, 1, fpprof);
    } //end for
    print("Profile Loaded\n");
  } // end if
  else
  {
    print("NEW PROFILE - Defaults Used (See Set Limits)\n");
  }
  fclose(fpprof);                             // Close the files


 //-------- Set Servos to rest position ------------------------------------------------------
  print("Reset Servos to Rest Pos\n");
  for(int i = 1; i <= 8; i++)                 // Counter for 8 servos; 8th is OK now
  {
    servo_angle((18-i), servorest[i]);        // Initialize all Servos to rest position
  } //end for



  //--------Main Menu Routine ---------------------------------------------------------------------
  while(1)
  {
    //------- Print Main Menu ---------------------------------------------------------------------
    print("  \n");
    print("----------- Main Menu -----------\n");
    print("File: %14s   Profile: %14s\n", servostr, profilestr);
    print("---------------------------------\n");
    print("      Manual         -  1\n");
    print("      Record Pot/Ch  -  2\n");
    print("      Playback       -  3\n");
    print("      Change File    -  4\n");
    print("      ADC Assign     -  5\n");
    print("      Set Limits     -  6\n");
    print("      New Show ADC   -  7\n");
    print("      Go to Rest     -  8\n");
    print("      Edit Line      -  9\n");

    print("  \n");
    print("Enter mode > ");
  
    scan("%d", &mode);                          //Get Mode from debug terminal keyboard (in decimal)



    if (mode == 1)              //Manual Mode -----------------------------
    {
      //-------- Manual mode - Practice with Servos ----------------------------------------
      print("\nManual Mode 4 Ch w/ Pot\n"); 
      print("Servo Bank Start (1 to 5)>");          // Get Servo Bank starting address
      status = scan("%d", &bankstart);
      if(bankstart > 5) bankstart = 5;                        // Limit Bank to 5
              
      print("Length of Test(sec)> ");               // Get Length of manual test
      status = scan("%d", &tmp); 
      if (tmp > 60) tmp = 60;                       // Limit Manual mode to 1 minute
      tmp = tmp * 20;                               // Time is 20 samples per second

      for(int i = 1; i <= (tmp); i++)               // Loop for user determined length
      {
         for(int j = bankstart; j <= bankstart+3; j++)    // AD convert, command, and store servo bank of 4 servos
         {
//            servocmd[j] = (adc_in(servoADC[j]) * 2 / 3);  // Read A/D# 0 thru 3 & convert 0 - 2700 A/D counts to 0 - 1800 range
                                                          // Corresponds to 0V to 3.3V input voltage
            servocmd[j] = ADCvalue[servoADC[j]];          // Get values from ADC cog running in background

            if(potrev[j] == 1)
            {
              servocmd[j] = 1800 - servocmd[j];           //invert parallax joystick vertical for pot # j
            }
            if(servocmd[j] <= servolow[j]) servocmd[j] = servolow[j];     // Limit cmd to low limit or greater (0 deg)
            if(servocmd[j] >= servohigh[j]) servocmd[j] = servohigh[j];   // Limit cmd to high limit or less (180 deg)

            servo_angle((18-j), servocmd[j]);            // Move selected Servo bank of 4 servos
         } // end for
         print("%5d\n", i);
         pause(40);                                      // Trim for 20HZ Update & new ADC cog in background  

      } //end for
 
    } //End If Mode 1




    else if (mode == 2)      //Record a servo channel with Potentiometer Mode ------------------

    {
      //-------- Record Servo Commands per channel via Potentiometer & Record to SD Card -----------------------
      print("\nCurrent File: %14s\n", servostr);
      print("Enter New Servo filename (.txt): ");
      getStr(servonewstr, 14);                 // Scan in string from debug terminal (have to hit enter) <= 14 char  

      print("\nEnter Servo# (0 for Bank))> ");
      status = scan("%d", &s);                         // Get servo number & status
      if (s > 8) s = 8;                                // Limit input to servo 8
      print("\nRecord file: %14s\n", servonewstr);   // Confirmation

      if (s == 0)                             // Check for Bank recording rather than channel recording
      {
        print("\nServo Bank Start (1 to 5)>");  // Get Servo Bank starting servo#
        status = scan("%d", &bankstart);
        if(bankstart > 5) bankstart = 5;      // Limit bankstart to 5
      }

      print("\nHit Enter to Start > ");                // Dummy input to get ready and start
      status = scan("%d", &n);                         

      fp = fopen(servostr, "r");                       // Open a file for reading back commands
      fptmp = fopen(servonewstr, "w");                 // Open new file for new servo commands

      print("\nReady... ");                            //Timer Indicator....
      pause(1000);
      print("Set... ");
      pause(1000);
      print("GO!\n");


      while(1)                                    // Run loop for variable amount until no more data in file
      {
        status = fread(&servocmd[1], 32, 1, fp);  // Read 32 Bytes (8 Servos) and assign to array 
        if (!status) break;                       // If end of file, break 

        for(int j = 1; j <= 8; j++)               // Check Limits of all 8 Servo cmd data - update in array
        {
          if(servocmd[j] < servolow[j]) servocmd[j] = servolow[j];    // Limit data to profile
          if(servocmd[j] > servohigh[j]) servocmd[j] = servohigh[j];     
        } ///end for


        if(s == 0)                                       // Bank recording
        {
          for(int s = bankstart; s <= bankstart+3; s++)
          {
            servocmd[s] = ADCvalue[servoADC[s]];         // Get values from ADC cog running in background
//            servocmd[s] = (adc_in(servoADC[s]) * 2 / 3); // Read A/D#3 & convert 0 - 2700 A/D counts to 0 - 1800 range
                                                         // Corresponds to 0V to 3.3V input voltage

            if(potrev[s] == 1)                           // Check for servo / pot reversal
            {
              servocmd[s] = 1800 - servocmd[s];          //invert parallax joystick vertical pot 1
            }

            if(servocmd[s] < servolow[s]) servocmd[s] = servolow[s];     // Limit n to low limit or greater (0 deg)
            if(servocmd[s] > servohigh[s]) servocmd[s] = servohigh[s];   // Limit n to high limit or less (180 deg)

          } // end for s
        } // end if

        else                                             // Single Channel Recording
        {
            servocmd[s] = ADCvalue[servoADC[s]];         // Get values from ADC cog running in background
//            servocmd[s] = (adc_in(servoADC[s]) * 2 / 3); // Read A/D#3 & convert 0 - 2700 A/D counts to 0 - 1800 range
                                                         // Corresponds to 0V to 3.3V input voltage

            if(potrev[s] == 1)                           // Check for servo / pot reversal
            {
              servocmd[s] = 1800 - servocmd[s];          //invert parallax joystick vertical pot 1
            }

            if(servocmd[s] < servolow[s]) servocmd[s] = servolow[s];     // Limit n to low limit or greater (0 deg)
            if(servocmd[s] > servohigh[s]) servocmd[s] = servohigh[s];   // Limit n to high limit or less (180 deg)
        } // end else




        fwrite(&servocmd[1], 32, 1, fptmp);     // Write to new file - 32 Bytes - All 8 servos      


        for(int j = 1; j <= 8; j++)              // Write all 8 servo commands to temp file on SD card and move servos
        {
          servo_angle((18-j), servocmd[j]);      // Move all Servos (including #8) 
        }  //end for

        pause(11);                                // Trimmed for 20HZ Update

      } //end while

      fclose(fp);                                 // Close the files
      fclose(fptmp);
      print("\nEnd of Record Pot\n");

      for(int i = 0; i <= 14; i++)                // Copy new filename into old filename (15 total char)
      {
        servostr[i] = servonewstr[i];             // Copy char by char 
      }

    }   // End Else Mode 2




    else if (mode == 3)            //Playback Mode ------------------------------

    {
      //----------------  SD Read and Playback -------------------------------
      print("\nPlayback File: %14s\n", servostr);  // List heading

      print("\nListing? (1-Yes  0-No) > ");  // Print out list of debug data
      status = scan("%d", &n);     

      fp = fopen(servostr, "r");                  // Open file for reading
      int i = 0;

      while(1)                                    // Run loop for variable amount until no more data in file
      {
        i++;                                      // Increment Record counter

        status = fread(&servocmd[1], 32, 1, fp);  // Read 32 Bytes (8 servos) at once and assign to array  
        if (!status) break;                       // If end of file, break 

        for(int j = 1; j <= 8; j++)               // Check limits of all 8 Servo cmd data - update array
        {
          if(servocmd[j] < servolow[j]) servocmd[j] = servolow[j];    // Limit data to profile
          if(servocmd[j] > servohigh[j]) servocmd[j] = servohigh[j];     
        } ///end for

//        if (!status) break;                     // If end of file, break

        for(int j = 1; j <= 8; j++)               // 8 servo commands to move servos
        {
          servo_angle((18-j), servocmd[j]);       // Move all Servos 
        }  //end for

        if (n == 1)                               // If n=1 then list debug output on screen; otherwise skip it
        {
          print("%5d: %5d %5d %5d %5d %5d %5d %5d %5d\n", i, servocmd[1], servocmd[2], servocmd[3], servocmd[4], servocmd[5], servocmd[6], servocmd[7], servocmd[8]); // Display value
        } //end if
        pause(38);                                // Trim for 20 Hz update rate with ADC cog in background; was 35

      } //end while 
      fclose(fp);                                 // Close the file
      print("Done Playback\n\n");
    } // End Else Mode 3
 



    else if (mode == 4)           //Change file name Mode------------------------------------

    {
      //------------------ Enter New Servo Command File Name & Profile Filename--------------------------------
      print("\nCurrent Servo File: %14s\n", servostr);
      print("Enter New Servo filename (.txt): ");
      getStr(servostr, 14);                      // Scan in string from debug terminal (have to hit enter) <= 14 char 

      while(1)
      {
        print("\nCurrent Profile File: %14s\n", profilestr);
        print("Enter New Profile filename (.txt): ");
        getStr(profilestr, 14);                 // Scan in string from debug terminal (have to hit enter) <= 14 char
        fpprof = fopen(profilestr, "r");        // Open a file for read
        if(fpprof) break;                       // if file exists (fpprof = 1) then proceed to load
        print("\nERROR - No File ...Try Again\n");
      } // end while

      for(int i = 1; i <= 8; i++)               //Readin Profile
      {
        fread(&servolow[i], 4, 1, fpprof);      // read profile from SD card New File
        fread(&servohigh[i], 4, 1, fpprof);
        fread(&servorest[i], 4, 1, fpprof);
        fread(&potrev[i], 4, 1, fpprof);
      } //end for

      print("Profile Loaded\n");
      fclose(fpprof);

    }  // End Else Mode 4




    else if (mode == 5)                   //ADC Assignment Mode --------------------------

    {
      //----------------  ADC Assignment Mode ---------------------------------------------
      print("  Servo     Assigned ADC\n");
      for (int i = 1; i <= 8; i++)
      {
        print("%5d         %5d\n",  i, servoADC[i]);
      }   


      print("\nEnter Servo# (Enter to exit)> ");
      status = scan("%d", &s);                      // Get servo number & status
 
      if (status)                                   // No changes...get out when status = 0
      {
        if (s > 8) s = 8;                           // Set Upper bound on servo #
        while(1)                                    // Set Lower Limit - Endless loop with BREAK
        {
          print("Enter ADC# (Enter to Exit)> ");
          status = scan("%d", &tmp);                // Get number & status
          if(!status) break;                        // Status 0? Break out of loop (2 enter presses)
          if(tmp <= 0) tmp = 0;                     // Limit to ADC 0 min
          if(tmp >= 3) tmp = 3;                     // Limit to ADC 3 max
        }  //end while
        servoADC[s] = tmp;                          // Assign ADC# to that servo
      }

    }  // End Else Mode 5





    else if (mode == 6)              //Set Servo Limits Mode ------------------------------

    {
      //------------------ Set Servo Limits -----------------------------------------------

      print("\nProfile: %14s\n", profilestr);
      print("\nServo#   Low     High     Rest    Pot Rev\n");
      for (int i = 1; i <=8; i++)
      {
        print(" %2d    %5d    %5d    %5d    %5d\n", i, servolow[i], servohigh[i], servorest[i], potrev[i]);
      }
      print("\nEnter Servo# (Enter to exit)> ");
      status = scan("%d", &n);                      // Get servo number & status
 
      if (status)                                   // No changes...get out when status = 0
      {
        if (n > 8) n = 8;                           // Set Upper bound on servo #
        while(1)                                    // Set Lower Limit - Endless loop with BREAK
        {
          print("Enter %d's low limit (Enter to Exit)>", n);
          status = scan("%d", &tmp);                // Get number & status
          if(!status) break;                        // Status 0? Break out of loop (2 enter presses)
          if(tmp <= 0) tmp = 0;                     // Limit n to low limit or greater (0 deg)
          if(tmp >= 1800) tmp = 1800;               // Limit n to high limit or less (180 deg)
          servo_angle((18-n), tmp);                 // Move Servo 1, 2, 3, 4, 5, 6, 7 and now 8 too

        }  //end while
        servolow[n] = tmp;                          // Assign lower limit to array
        print("Stored for Servo %d\n\n", n);


        while(1)                                    // Set Upper Limit - Endless loop with BREAK
        {
        print("Enter %d's up limit (Enter to Exit)>", n);
          status = scan("%d", &tmp);                // Get number & status
          if(!status) break;                        // Status 0? Break out of loop (2 enter presses)
          if(tmp <= 0) tmp = 0;                     // Limit n to low limit or greater (0 deg)
          if(tmp >= 1800) tmp = 1800;               // Limit n to high limit or less (180 deg)
          servo_angle((18-n), tmp);                 // Move Servo 1, 2, 3, 4, 5, 6, 7... and now 8 too
        
        }  //end while
        servohigh[n] = tmp;                         // Assign upper limit to array
        print("Stored for Servo %d\n\n", n);


        while(1)                                    // Set Rest Position - Endless loop with BREAK
        {
        print("Enter %d's rest pos (Enter to Exit)>", n);
          status = scan("%d", &tmp);                // Get number & status
          if(!status) break;                        // Status 0? Break out of loop (2 enter presses)
          if(tmp <= 0) tmp = 0;                     // Limit n to low limit or greater (0 deg)
          if(tmp >= 1800) tmp = 1800;               // Limit n to high limit or less (180 deg)
          servo_angle((18-n), tmp);                 // Move Servo 1, 2, 3, 4, 5, 6, 7... and now 8 too

        }  //end while
        servorest[n] = tmp;                         // Assign rest position to array
        print("Stored for Servo %d\n\n", n);


        while(1)                                    // Set Rest Position - Endless loop with BREAK
        {
        print("Enter %d's Pot Rev Flag 0=Norm 1=Rev (Enter to Exit)>", n);
          status = scan("%d", &tmp);                // Get number & status
          if(!status) break;                        // Status 0? Break out of loop (2 enter presses)
          if(tmp <= 0) tmp = 0;                     // Limit n to 0
          if(tmp >= 1) tmp = 1;                     // Limit n to 1
        }  //end while

        potrev[n] = tmp;                            // Assign Pot Reversal Flag to array
        print("Stored for Servo %d\n\n", n);

      

        //--------- Save off to Profile ----------------------
        print("\nCurrent Profile File: %14s\n", profilestr);
        print("Enter New Profile filename (.txt): ");
        getStr(profilestr, 14);                     // Scan in string from debug terminal (have to hit enter) <= 14 char
        print("Confirm: %14s\n", profilestr);

        fpprof = fopen(profilestr, "w");            // Open a file for writing
    
        for(int i = 1; i <= 8; i++)
        {
          fwrite(&servolow[i], 4, 1, fpprof);       // Write servo profile to SD card New File
          fwrite(&servohigh[i], 4, 1, fpprof);
          fwrite(&servorest[i], 4, 1, fpprof);
          fwrite(&potrev[i], 4, 1, fpprof);
        } //end for

        fclose(fpprof);                             // Close the files
        print("Profile Saved\n");
      }  //end if


    }  // End Else Mode 6




    else if (mode == 7)      //New Show Creation - Record with Potentiometers  qty 4 -----------------------------

    {
      //-------- Create New Show, file, length, servo commands via Potentiometer & Record to SD Card ----------------
      print("\nCreate New 8 ch Show & Default file\n");
      print("\nCurrent File: %14s\n", servostr);

      print("Enter New Servo filename (.txt): ");
      getStr(servostr, 14);                        // Scan in string from debug terminal (have to hit enter) <= 14 char

      print("Servo Bank Start (1 to 5)>");         // Get Servo Bank starting servo#
      status = scan("%d", &bankstart);
      if(bankstart > 5) bankstart = 5;             // Limit bankstart to 5

      print("\nEnter Show Time (50mS sample)> ");
      scan("%d", &showtime);                       //Get show time from debug terminal keyboard (in decimal)

      //----- Pre Load the servo command array with rest positions
      for(int j = 1; j <= 8; j++)
      {
        servocmd[j] = servorest[j];                // Set default rest points for all 8 servos
      } //end for

      fp = fopen(servostr, "w");                   // Open a file for writing

      print("\nHit Enter to Start > ");            // Dummy input to get ready and start
      status = scan("%d", &n);                         

      print("\nReady... ");                        //Timer Indicator....
      pause(1000);
      print("Set... ");
      pause(1000);
      print("GO!\n");

      for(int i = 1; i <= showtime; i++)           // Run loop for variable amount of 50mS samples - showtime length
      {
         for(int j = bankstart; j <= bankstart+3; j++)      // AD convert, command, and store to servo bank
         {
            servocmd[j] = ADCvalue[servoADC[j]];            // Get values from ADC cog running in background
//            servocmd[j] = (adc_in(servoADC[j]) * 2 / 3);  // Read A/D# 0 thru 3 & convert 0 - 2700 A/D counts to 0 - 1800 range
                                                            // Corresponds to 0V to 3.3V input voltage
            if(potrev[j] == 1)
            {
              servocmd[j] = 1800 - servocmd[j];             //invert parallax joystick vertical for pot # j
            }
            if(servocmd[j] <= servolow[j]) servocmd[j] = servolow[j];     // Limit cmd to low limit or greater (0 deg)
            if(servocmd[j] >= servohigh[j]) servocmd[j] = servohigh[j];   // Limit cmd to high limit or less (180 deg)

            servo_angle((18-j), servocmd[j]);          // Move selected Servo bank
         } // end for


        //------ Write new file in series of 8 records with selected servo commands and the rest default positions ------------------

        fwrite(&servocmd[1], 32, 1, fp);            // Write 32 Bytes (all 8 servos) SD card & rest points to all 8 servos

        pause(14);                                  // Trim for 20HZ Update & new ADC cog in background; was 13mS

      } //end for

      fclose(fp);                                   // Close the file
      print("\nEnd of New Show Record");

    }   // End Else Mode 7




    else if (mode == 8)      //Command Reset Positions to all Servos  ------------------------------

    {
      //-------- Set Servos to rest position ------------------------------------------------------
      print("Reset Servos to Rest Pos\n");
      for(int i = 1; i <= 8; i++)                   // Counter for 8 servos; 8th is OK now
      {
        servo_angle((18-i), servorest[i]);          // Initialize all Servos to rest position
      } //end for

      pause(250);

    }   // End Else Mode 8



    else if (mode = 9)      //Line Edit Mode ---------------------------------------
    {
      //----------------  SD Read and prinout lines of data -------------------------------
      print("\nPlayback File: %14s\n", servostr); // List heading

      print("\n Start Listing at > ");            // Input start sample of Print out list of debug data - 20 lines
      status = scan("%d", &n);  
      if(n < 1) n = 1;

      int bytecount = (n - 1) * 8 * 4;            // no. of bytes to count into SD card file (8 samples * 4 Bytes/sample / servo sample)

      print("\nSample   S1    S2    S3    S4    S5    S6    S7    S8\n");  //Header   

      fp = fopen(servostr, "r");                  // Open file for reading
      rewind(fp);                                 // Rewind SD card file pointer to beginning of file
      fseek(fp, bytecount, SEEK_CUR);             // Move file pointer ahead to bytecount offset from current fp position

      for (int i = n; i <= n + 20; i++)           // Run loop for 20 lines of data in file
      {
        for(int j = 1; j <= 8; j++)               // Read all 8 Servo cmd data - Store in array
        {
          status = fread(&servocmd[j], 4, 1, fp);   
        } ///end for

        if (!status) break;                       // If end of file, break
        print("%5d: %5d %5d %5d %5d %5d %5d %5d %5d\n", i, servocmd[1], servocmd[2], servocmd[3], servocmd[4], servocmd[5], servocmd[6], servocmd[7], servocmd[8]); // Display value
        pause(35);                                // pause a little
      } //end for 
      fclose(fp);                                 // Close the file
      print("Done Printout\n\n");


     //--------- Edit Portion -------------------------------
      print("* WARNING - Will Modify File *\n");
      print("\n  Enter 5 to Overwrite -or- Enter nothing to Exit > ");
      status = scan("%d", &n);
      if(n == 5)                                    // Provide Exit criteria - Enter 5 to Overwrite, Just enter to escape
      {  
        print("\n Select Sample Line, Servo, Value > ");   // Input Line to edit
        int v = 0;
        status = scan("%d  %d  %d", &n, &s, &v);  
        if (n == 0) break;                          // Null input... get out of mode
        if(n < 1) n = 1;

        s = (s - 1) * 4;
        bytecount = (n - 1) * 8 * 4;                // no. of bytes via sample count into SD card file (8 samples * 4 Bytes/sample / servo sample)


        fp = fopen(servostr, "r+");                 // Open file for reading and writing
        rewind(fp);                                 // Rewind SD card file pointer to beginning of file
        fseek(fp, bytecount + s, SEEK_CUR);         // Move file pointer ahead to bytecount offset from current fp position
        fwrite(&v, 4, 1, fp);                       // Update that servo position in file with new value
        fclose(fp);                                 // Close the file

        fp = fopen(servostr, "r");                  // Open file for reading
        rewind(fp);                                 // Rewind SD card file pointer to beginning of file
        fseek(fp, bytecount, SEEK_CUR);             // Move file pointer ahead to bytecount offset


        print("Sample   S1    S2    S3    S4    S5    S6    S7    S8\n");  //Header   
        for(int j = 1; j <= 8; j++)                 // Read all 8 Servo cmd data - Store in array
        {
          status = fread(&servocmd[j], 4, 1, fp);   
        } ///end for
        if (!status) break;                         // If end of file, break

        print("%5d: %5d %5d %5d %5d %5d %5d %5d %5d\n", n, servocmd[1], servocmd[2], servocmd[3], servocmd[4], servocmd[5], servocmd[6], servocmd[7], servocmd[8]); // Display value
        pause(35);                                  // pause a little
        fclose(fp);                                 // Close the file
      } //End If

    } // end else mode 9


 
  }  // End While
} // End Main   






//---------- FUNCTION DEFINITIONS ---------------------------------------------------------
void ADCbackground()                          // ADC Function in other cog
{
  adc_init(21, 20, 19, 18);                   // CS=21, SCL=20, DO=19, DI=18

  while(1)                                    // Endless loop
  {
    ADCvalue[0] = (adc_in(0) * 2 / 3);        // Read A/D#0 & convert 0 - 2700 A/D counts to 0 - 1800 range
                                              // Corresponds to 0V to 3.3V input voltage
    ADCvalue[1] = (adc_in(1) * 2 / 3);        // Read A/D#1 & convert 0 - 2700 A/D counts to 0 - 1800 range
                                              // Corresponds to 0V to 3.3V input voltage
    ADCvalue[2] = (adc_in(2) * 2 / 3);        // Read A/D#2 & convert 0 - 2700 A/D counts to 0 - 1800 range
                                              // Corresponds to 0V to 3.3V input voltage
    ADCvalue[3] = (adc_in(3) * 2 / 3);        // Read A/D#3 & convert 0 - 2700 A/D counts to 0 - 1800 range                                             // Corresponds to 0V to 3.3V input voltage
  }
}